using System.Security.Cryptography;
using Hydra.Backend.Database;
using Hydra.Backend.Database.Models;
using Hydra.Backend.Utils;
using Microsoft.EntityFrameworkCore;
using NodaTime;

namespace Hydra.Backend.Middleware;

public class AuthenticationMiddleware(HydraContext db, IClock clock) : IMiddleware
{
    public async Task InvokeAsync(HttpContext ctx, RequestDelegate next)
    {
        var endpoint = ctx.GetEndpoint();
        var metadata = endpoint?.Metadata.GetMetadata<AuthenticateAttribute>();

        if (metadata == null)
        {
            await next(ctx);
            return;
        }

        var header = ctx.Request.Headers.Authorization.ToString();
        if (!AuthUtils.TryFromBase64String(header, out var rawToken))
        {
            await next(ctx);
            return;
        }

        var hash = SHA512.HashData(rawToken);
        var oauthToken = await db.Tokens
            .Include(t => t.Application)
            .Include(t => t.Account)
            .FirstOrDefaultAsync(t => t.Hash == hash && t.ExpiresAt > clock.GetCurrentInstant() && !t.ManuallyExpired);
        if (oauthToken == null)
        {
            await next(ctx);
            return;
        }

        ctx.SetToken(oauthToken);

        await next(ctx);
    }
}

public static class HttpContextExtensions
{
    private const string Key = "token";

    public static void SetToken(this HttpContext ctx, Token token) => ctx.Items.Add(Key, token);
    public static Account? GetAccount(this HttpContext ctx) => ctx.GetToken()?.Account;

    public static Account GetAccountOrThrow(this HttpContext ctx) =>
        ctx.GetAccount() ?? throw new ApiError.AuthenticationError("No account in HttpContext");

    public static Token? GetToken(this HttpContext ctx)
    {
        if (ctx.Items.TryGetValue(Key, out var token))
            return token as Token;
        return null;
    }
}

[AttributeUsage(AttributeTargets.Class | AttributeTargets.Method)]
public class AuthenticateAttribute : Attribute;